# sb::Plugin::TemplateWizard
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

use strict;

# ======================================
package sb::Plugin::TemplateWizard;
# --------------------------------------
# // configuration for inheritance / dependancy
use sb::Plugin ();
# --------------------------------------
# // initialization for plugin
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin, receipt',
		'name'    => 'Template Wizard',
		'text'    => 'Template Wizard plugin for Serene Bach',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => version(),
	},
	'file' => 'tempwiz.txt',
	'data' => 1,
);
sb::Plugin->register_admin_module(
	'mode'   => 'tempwiz',
	'level'  => 1,
	'module' => 'sb::Admin::TemplateWizard',
);
sub version
{
	return '0.05';
	# 0.05 [2008/05/19] fixed a javascript bug
	# 0.04 [2008/03/06] first release
	# 0.03 [2007/03/12] changed color
	# 0.02 [2007/03/11] added sample for color setting
	# 0.01 [2007/03/10] first beta
	# 0.00 [2007/02/15] generated
}
# ======================================
package sb::Admin::TemplateWizard;
# --------------------------------------
# // declaration for global variables
use vars qw( @ISA );
# --------------------------------------
# // configuration for inheritance / dependancy
use sb::Config ();
use sb::Language ();
use sb::Interface ();
use sb::Data::Image ();
use sb::Admin::Template ();
@ISA = qw( sb::Admin::Template );
# --------------------------------------
# // declaration for constant value
sub TEMPLATE        (){ 'tempwiz.html' };
sub DEFAULT_TYPE    (){ 'base' };
sub EXTRA_CSSPREFIX (){ 'ext_' };
sub EXTRA_STYLE     (){ '<link rel="stylesheet" type="text/css" href="%s" title="default" media="screen,tv" />' };
sub ADDITIONAL_CSS  (){ "<style type=\"text/css\">\n%s\n</style>" };
sub IMAGE_FILEFORM  (){ '<input type="file" name="%s" id="%s" value="" size="40" />' };
sub IMAGE_DEL_FORM  (){ '[%s] <label>%s <input type="checkbox" name="del_%s" id="%s" value="on" /></label>' };
sub IMAGE_PREVIEW   (){ '<dd class="form_option"><img src="%s" width="%d" height="%d" alt="" /></dd>' };
sub JS_SUBMIT       (){ ' onsubmit="selectAllTarget()"' };
sub IMAGE_MAX_WIDTH (){ 360 };
sub MIN_MARGIN      (){ 30 };
my @aWizTypes = ('base','color','image');
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'temp'} = undef;
	$self->{'data'} = sb::Plugin->get_data;
	if ($self->{'data'}->setting ne '')
	{
		$self->{'temp'} = sb::Data->load('Template','id'=>$self->{'data'}->setting);
	}
	return ( $self->{'regi'} ) 
		? $self->_save_template(@_)
		: $self->_open_template(@_);
}
# --------------------------------------
# // public functions - override
sub common_template_parts
{
	my $self = shift;
	my $cms = shift;
	my $lang = sb::Language->get;
	return if (!$cms or !$self);
	$self->SUPER::common_template_parts($cms);
	my $css = '';
	if ($self->{'user'} and $self->{'user'}->ad_css ne '')
	{
		my $ext = sb::Config->get->value('srv_temp') . $lang->code . '/' . EXTRA_CSSPREFIX;
		$css .= "\n" . sprintf(EXTRA_STYLE,$ext . $self->{'user'}->ad_css . '.css');
	}
	$css .= "\n" . sprintf(ADDITIONAL_CSS,$self->_additinal_style());
	$cms->num(0);
	$cms->tag('sb_extra_style'=>$css);
}
# --------------------------------------
# // private functions - main routines
sub _save_template
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi = sb::Interface->get;
	if ($self->{'regi'} eq 'new')
	{
		$self->_create_newtemplate($cgi);
	}
	elsif ($self->{'regi'} eq 'base')
	{
		$self->_update_base($cgi);
	}
	elsif ($self->{'regi'} eq 'color')
	{
		$self->_update_color($cgi);
	}
	elsif ($self->{'regi'} eq 'image')
	{
		$self->_update_image($cgi);
	}
	sb::Plugin->set_data('data'=>$self->{'data'});
	$self->_update_template() if ($self->{'temp'});
	return $self->_open_template('message'=>sb::Language->get->string('parts_editcomp'));
}
sub _open_template
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cms = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $type = $self->_check_type(sb::Interface->get->value('__type'));
	if ($self->{'temp'})
	{
		my $viewpath = sb::Config->get->value('conf_srv_cgi') . sb::Config->get->value('basic_sb');
		my $func = '_handle_' . $type;
		$self->$func($cms);
		$cms->num(0);
		$cms->tag('sb_tempwiz_viewlink' => $viewpath . '?tid=' . $self->{'temp'}->id);
		$cms->block('sb_tempwiz_edit' => 1);
		$cms->block('sb_tempwiz_' . $type => 1);
	}
	$cms->num(0);
	$cms->tag('sb_tempwiz_menu_' . $type => 'class="current"');
	$cms->tag('sb_tempwiz_type' => $type);
	if ($param{'message'} ne '')
	{
		$cms->num(0);
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_process_message'=>1);
	}
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}
# --------------------------------------
# // private functions - controller
sub _create_newtemplate
{
	my $self = shift;
	my $cgi = shift;
	$self->{'temp'} = undef;
	my $new = sb::Data->add('Template',
		'use'   => 0,
		'name'  => sb::Language->get->string('tempwiz_template_name'),
		'gen'   => $self->{'time'},
		'mod'   => $self->{'time'},
		'main'  => '',
		'css'   => '',
		'entry' => '',
		'info'  => '',
	);
	if ($new)
	{
		$self->{'temp'} = $new;
		$self->{'data'}->setting($new->id);
		$self->_set_base($self->_default_base());
		$self->_set_color($self->_default_color());
		$self->_set_image($self->_default_image());
	}
}
sub _update_base
{
	my $self = shift;
	my $cgi = shift;
	my $base = $self->_get_base();
	# --- update column
	my $column = $cgi->value('tempwiz_column');
	if ($column eq 'both' or $column eq 'left' or $column eq 'right')
	{
		$base->{'column'} = $cgi->value('tempwiz_column');
	}
	# --- calender and width
	$base->{'calendar'} = ($cgi->value('tempwiz_calendar') eq 'on') ? 'on' : '';
	my $basewidth = int($cgi->value('tempwiz_basewidth'));
	my $sidewidth = int($cgi->value('tempwiz_sidewidth'));
	if ($basewidth > $sidewidth + MIN_MARGIN and $sidewidth > 0)
	{
		$base->{'content_width'} = $basewidth;
		$base->{'sidebar_width'} = $sidewidth;
	}
	# --- sidebar settings
	my @candidates = $self->_get_sidebar_list();
	for my $target ('left','center','right')
	{
		my @array = ();
		for my $list ( split("\0",$cgi->value('tempwiz_' . $target . 'list')) )
		{
			push(@array,$list) if ( grep(/\Q$list\E/,@candidates) );
		}
		$base->{$target} = \@array;
	}
	$self->_set_base($base);
}
sub _update_color
{
	my $self = shift;
	my $cgi = shift;
	my $color = $self->_get_color();
	for my $key ( $self->_get_color_list() )
	{
		my $newcolor = $cgi->value('tempwiz_color_' . $key);
		$color->{$key} = $newcolor if ($newcolor =~ /^[0-9a-f]{3,6}$/i);
	}
	$self->_set_color($color);
}
sub _update_image
{
	my $self = shift;
	my $cgi = shift;
	my $image = $self->_get_image();
	for my $key ( $self->_get_image_list() )
	{
		my %check = (
			'x'   => ['left','center','right'],
			'y'   => ['top','center','bottom'],
			'rep' => ['no-repeat','repeat-x','repeat-y','repeat'],
		);
		for my $image_key ( keys %check )
		{
			my $value = $cgi->value('tempwiz_image_' . $key . '_' . $image_key);
			if ( grep(/^\Q$value\E$/, @{$check{$image_key}}) )
			{
				$image->{$key}->{$image_key} = $value;
			}
		}
		if ($cgi->value('del_tempwiz_image_' . $key) eq 'on')
		{
			my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
			my $file = $dir . $image->{$key}->{'name'};
			unlink($file) if (-e $file);
			$image->{$key}->{'name'} = '';
		}
	}
	my $upload = $self->_upload($cgi);
	for my $name ( keys %$upload )
	{
		$image->{$name}->{'name'} = $upload->{$name};
	}
	$self->_set_image($image);
}
# --------------------------------------
# // private functions - view handler
sub _handle_base
{
	my $self = shift;
	my $cms = shift;
	my $lang = sb::Language->get();
	my $base = $self->_get_base();
	$cms->num(0);
	$cms->tag('sb_tempwiz_img_colleft'  => sb::Plugin->get_resource_dir() . 'column_left.gif');
	$cms->tag('sb_tempwiz_img_colright' => sb::Plugin->get_resource_dir() . 'column_right.gif');
	$cms->tag('sb_tempwiz_img_colboth'  => sb::Plugin->get_resource_dir() . 'column_both.gif');
	$cms->tag('sb_tempwiz_radio_' . $base->{'column'} => 'checked="checked"');
	$cms->tag('sb_tempiz_checkbox_cal' => 'checked="checked"') if ($base->{'calendar'} eq 'on');
	$cms->tag('sb_tempwiz_basewidth'=>$base->{'content_width'});
	$cms->tag('sb_tempwiz_sidewidth'=>$base->{'sidebar_width'});
	for my $side ('left','center','right')
	{
		my $option = '';
		for my $item ( @{$base->{$side}} )
		{
			$option .= '<option value="' . $item . '">' . $lang->string('tempwiz_sidebarname_' . $item) . '</option>';
		}
		$cms->tag('sb_tempwiz_' . $side . 'list' => $option);
	}
	$cms->tag('sb_tempwiz_submit' => JS_SUBMIT);
}
sub _handle_color
{
	my $self = shift;
	my $cms = shift;
	my $lang = sb::Language->get();
	my $color = $self->_get_color();
	my $num = 0;
	for my $key ( $self->_get_color_list() )
	{
		$cms->num($num);
		$cms->tag('color_name'=>$key);
		$cms->tag('color_code'=>$color->{$key});
		$cms->tag('color_label'=>$lang->string('tempwiz_color_' . $key));
		$cms->tag('color_set'=>join(',', map { "'$_'" } $self->_get_color_set($key)));
		$cms->tag('color_type'=>$lang->string('tempwiz_colortype_border')) if ($key =~ /border/);
		$cms->tag('color_type'=>$lang->string('tempwiz_colortype_background')) if ($key =~ /background/);
		$num++;
	}
	$cms->block('sb_tempwiz_color_list'=>$num);
	{ # preset colors
		my $preset = '';
		my @check = $self->_get_color_set( ($self->_get_color_list())[0] );
		for my $i (0..$#check)
		{
			my $label = 'preset_color' . $i;
			my $string = $lang->string('tempwiz_' . $label);
			$string = $label if ($string eq '');
			$preset .= '<option value="' . $i . '">' . $string . '</option>';
		}
		$cms->num(0);
		$cms->tag('preset_colors'=>$preset);
	}
	$cms->num(0);
	$cms->tag('sb_tempwiz_parts'=>sb::Plugin->get_resource_dir());
}
sub _handle_image
{
	my $self = shift;
	my $cms = shift;
	my $lang = sb::Language->get();
	my $image = $self->_get_image();
	my $num = 0;
	for my $key ( $self->_get_image_list() )
	{
		my $label = 'tempwiz_image_' . $key;
		$cms->num($num);
		if ($image->{$key}->{'name'} eq '')
		{
			$cms->tag('image_file_form'=>sprintf(IMAGE_FILEFORM,$label,$label));
		}
		else
		{
			my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
			my $name = $image->{$key}->{'name'};
			$cms->tag('image_file_form'=>sprintf(IMAGE_DEL_FORM,$name,$lang->string('tempwiz_delete_image'),$label,$label));
			if (-e $dir . $name)
			{
				my ($w,$h) = $self->_check_size($dir . $name);
				my $base = sb::Config->get->value('conf_srv_base') . sb::Config->get->value('dir_style');
				$cms->tag('image_current'=>sprintf(IMAGE_PREVIEW,$base . $name,$w,$h));
			}
		}
		my $rep = $image->{$key}->{'rep'};
		$rep =~ s/\-//g;
		$cms->tag('sb_tempwiz_imgx_' . $image->{$key}->{'x'} => 'selected="selected"');
		$cms->tag('sb_tempwiz_imgy_' . $image->{$key}->{'y'} => 'selected="selected"');
		$cms->tag('sb_tempwiz_imgrep_' . $rep => 'selected="selected"');
		$cms->tag('image_name'=>$key);
		$cms->tag('image_label'=>$lang->string($label));
		$num++;
	}
	$cms->block('sb_tempwiz_image_list'=>$num);
}
# --------------------------------------
# // private functions - plugin data
sub _get_base
{ # $self->{'data'}->data stores base information
	my $self = shift;
	my %info = ();
	my @data = split(',',$self->{'data'}->data());
	for my $key ( $self->_get_basedata_list() )
	{
		$info{$key} = shift(@data);
	}
	for my $key ('left','center','right')
	{
		my @array = split(':',$info{$key});
		$info{$key} = \@array;
	}
	return \%info;
}
sub _set_base
{
	my $self = shift;
	my $base = shift;
	my $data = '';
	for my $key ( $self->_get_basedata_list() )
	{
		if ($key eq 'left' or $key eq 'center' or $key eq 'right')
		{
			$data .= join(':',@{$base->{$key}}) . ',';
		}
		else
		{
			$data .= $base->{$key} . ','
		}
	}
	$self->{'data'}->data($data);
}
sub _default_base
{
	my $self = shift;
	my %info = (
		'column'        => 'right',
		'calendar'      => '',
		'left'          => [],
		'center'        => [$self->_get_sidebar_list()],
		'right'         => [],
		'content_width' => 780,
		'sidebar_width' => 170,
	);
	return \%info;
}
sub _get_color
{ # $self->{'data'}->text stores color information
	my $self = shift;
	my %info = ();
	my @data = split(',',$self->{'data'}->text());
	for my $key ( $self->_get_color_list() )
	{
		$info{$key} = shift(@data);
	}
	return \%info;
}
sub _set_color
{
	my $self = shift;
	my $base = shift;
	my $data = '';
	for my $key ( $self->_get_color_list() )
	{
		$data .= $base->{$key} . ',';
	}
	$self->{'data'}->text($data);
}
sub _default_color
{
	my $self = shift;
	my %info = ();
	for my $color ( $self->_get_color_list() )
	{
		$info{$color} = ($self->_get_color_set($color))[0];
	}
	return \%info;
}
sub _get_image
{ # $self->{'data'}->extra stores image information
	my $self = shift;
	my %info = ();
	my @data = split(',',$self->{'data'}->extra());
	for my $key ( $self->_get_image_list() )
	{
		my $value = shift(@data);
		my @details = split(':',$value);
		$info{$key} = {
			'name' => $details[0],
			'x'    => $details[1],
			'y'    => $details[2],
			'rep'  => $details[3],
		};
	}
	return \%info;
}
sub _set_image
{
	my $self = shift;
	my $base = shift;
	my $data = '';
	for my $image ( $self->_get_image_list() )
	{
		$data .= $base->{$image}->{'name'} . ':';
		$data .= $base->{$image}->{'x'} . ':';
		$data .= $base->{$image}->{'y'} . ':';
		$data .= $base->{$image}->{'rep'} . ',';
	}
	$self->{'data'}->extra($data);
}
sub _default_image
{
	my $self = shift;
	my %info = ();
	for my $image ( $self->_get_image_list() )
	{
		$info{$image} = {
			'name' => '',
			'x'    => 'left',
			'y'    => 'top',
			'rep'  => 'repeat',
		};
	}
	return \%info;
}
# --------------------------------------
# // private functions - utilities
sub _update_template
{
	my $self = shift;
	my $temp = $self->{'temp'};
	return if (!$temp);
	my $base = $self->_get_base();
	$temp->main($self->_create_basehtml($base));
	$temp->css($self->_create_css($base));
	$temp->info($self->_template_info());
	sb::Data->update($temp);
}
sub _create_basehtml
{
	my $self = shift;
	my $base = shift;
	my $html = sb::TemplateManager->new($self->_template_htmlbase());
	$html->num(0);
	if ($base->{'calendar'})
	{
		$html->tag('tempwiz_horizontal_calendar'=>$self->_template_horizontal_calendar());
	}
	if ($base->{'column'} eq 'left' or $base->{'column'} eq 'right')
	{
		my $navi = '';
		for my $navi_type ( @{$base->{$base->{'column'}}} )
		{
			my $func = '_template_' . $navi_type;
			eval {
				$navi .= $self->$func();
			};
		}
		$navi .= $self->_template_other();
		$html->tag('tempwiz_mainnavi'=>$self->_template_navi('navigation',$navi));
		$html->tag('tempwiz_subnavi'=>'');
	}
	elsif ($base->{'column'} eq 'both')
	{
		my $navi = '';
		for my $navi_type ( @{$base->{'left'}} )
		{
			my $func = '_template_' . $navi_type;
			eval {
				$navi .= $self->$func();
			};
		}
		$html->tag('tempwiz_subnavi'=>$self->_template_navi('navi1st',$navi));
		$navi = '';
		for my $navi_type ( @{$base->{'right'}} )
		{
			my $func = '_template_' . $navi_type;
			eval {
				$navi .= $self->$func();
			};
		}
		$navi .= $self->_template_other();
		$html->tag('tempwiz_mainnavi'=>$self->_template_navi('navi2nd',$navi))
	}
	my $output = $html->output();
	$output =~ s/\[/\{/g;
	$output =~ s/\]/\}/g;
	$output =~ s/<!--%/<!-- /g;
	$output =~ s/%-->/ -->/g;
	$output;
}
sub _create_css
{
	my $self = shift;
	my $base = shift;
	my $css_base = $self->_template_cssbase();
	$css_base .= $self->_template_css_2columns() if ($base->{'column'} ne 'both');
	$css_base .= $self->_template_css_3columns() if ($base->{'column'} eq 'both');
	$css_base .= $self->_create_css_color();
	$css_base .= $self->_create_css_image();
	my $css = sb::TemplateManager->new($css_base);
	$css->num(0);
	$css->tag('body_width'=>$base->{'content_width'});
	$css->tag('navi_width'=>$base->{'sidebar_width'});
	if ($base->{'column'} ne 'both')
	{
		my $content_width = $base->{'content_width'} - $base->{'sidebar_width'} - 20;
		$css->tag('content_float' => ($base->{'column'} eq 'left') ? 'right' : 'left');
		$css->tag('navi_float' => ($base->{'column'} eq 'left') ? 'right' : 'left');
		$css->tag('content_width'  => $content_width);
		$css->tag('navi_margin'    => $content_width + 10);
		$css->tag('navi_margin_ie' => $content_width + 5);
	}
	else
	{
		my $content_width = $base->{'content_width'} - ($base->{'sidebar_width'} * 2) - 5;
		$css->tag('content_width' => $content_width);
		$css->tag('navi_width'    => $base->{'sidebar_width'} - 5);
		$css->tag('navi_margin'   => $content_width + $base->{'sidebar_width'} + 5);
	}
	$css->tag('cal_width'=>int($base->{'sidebar_width'} / 8));
	$css->tag('site_parts'=>'{site_parts}');
	return $css->output();
}
sub _create_css_color
{
	my $self = shift;
	my $css = "/* [colors] */\n";
	my $color = $self->_get_color();
	for my $tag ( $self->_get_color_list() )
	{
		my $temp = $self->_template_css_color($tag);
		my $label = join(",\n",$self->_get_css_label($tag));
		my $code = $color->{$tag};
		$temp =~ s/\%css_tag\%/$label/g;
		$temp =~ s/\%color_code\%/$code/g;
		$css .= $temp;
	}
	return $css;
}
sub _create_css_image
{
	my $self = shift;
	my $css = "/* [images] */\n";
	my $image = $self->_get_image();
	for my $tag ( $self->_get_image_list() )
	{
		next if ($image->{$tag}->{'name'} eq '');
		my $temp = $self->_template_css_image($tag);
		my $label = join(",\n",$self->_get_css_label($tag));
		my $name = $image->{$tag}->{'name'};
		my $x    = $image->{$tag}->{'x'};
		my $y    = $image->{$tag}->{'y'};
		my $rep  = $image->{$tag}->{'rep'};
		$temp =~ s/\%css_tag\%/$label/g;
		$temp =~ s/\%img_x\%/$x/g;
		$temp =~ s/\%img_y\%/$y/g;
		$temp =~ s/\%img_repeat\%/$rep/g;
		$temp =~ s/\%img_name\%/$name/g;
		$css .= $temp;
	}
	return $css;
}
sub _check_type
{
	my $self = shift;
	my $type = shift;
	$type = DEFAULT_TYPE if ($type eq '');
	$type = DEFAULT_TYPE if ( !grep(/^\Q$type\E$/,@aWizTypes) );
	return $type;
}
sub _output
{
	my $self = shift;
	my $data = shift;
	$data =~ s/^\s+//;
	$data =~ s/\n\s+/\n/g;
	$data =~ s/^#\s//;
	$data =~ s/\n#\s/\n/g;
	$data;
}
sub _upload
{
	my $self = shift;
	my $cgi = shift;
	my $dir = sb::Config->get->value('conf_dir_base') . sb::Config->get->value('dir_style');
	my @forms = ();
	my %files = ();
	for my $name ( $self->_get_image_list() )
	{
		push(@forms,$name) if ($cgi->value('tempwiz_image_' . $name) ne '');
	}
	my @header = $cgi->content_list();
	for my $i (0..$#forms)
	{
		my $entity = $cgi->value('tempwiz_image_' . $forms[$i]);
		my $label  = 'tempwiz_image_' . $forms[$i];
		my %param  = ();
		foreach (@header)
		{
			next if ($_ !~ /$label/);
			$param{'type'}     = $2 if ($_ =~ /(.*)Content-type:(.*)/i);
			$param{'filename'} = $2 if ($_ =~ /(.*)filename=(.*)/i);
			$param{'macbin'}   = ($_ =~ /application\/x-macbinary/i) ? 1 : undef;
			last;
		}
		if ($param{'macbin'})
		{ # handle mac binary
			my $len = substr($entity,83,4);
			$len = unpack("%N",$len);
			$entity = substr($entity,128,$len);
		}
		$param{'type'} =~ tr/\x0D\x0A//d;
		$param{'type'} =~ s/\s//g;
		$param{'filename'} =~ tr/\"\x0D\x0A//d;
		$param{'filename'} =~ s/.*[:\/\\](.*)/$1/; # extract file name only
		next if ($param{'filename'} =~ /[^a-zA-Z0-9_\-\.]/);
		next if (-e $dir . $param{'filename'});
		my $file = $dir . $param{'filename'};
		eval
		{
			open(BINOUT,">$file") or die('failed file open');
			binmode(BINOUT);
			print BINOUT $entity;
			close(BINOUT);
			chmod(sb::Config->get->value('basic_file_attr'),$file);
		};
		next if ($@);
		$files{$forms[$i]} = $param{'filename'};
	}
	return \%files;
}
sub _check_size
{
	my $self = shift;
	my $file = shift;
	my ($w,$h) = 
		  ($file =~ /\.gif$/i)   ? &sb::Data::Image::_get_size_gif($file)
		: ($file =~ /\.jpe?g$/i) ? &sb::Data::Image::_get_size_jpg($file)
		: ($file =~ /\.png$/i)   ? &sb::Data::Image::_get_size_png($file)
		: ($file =~ /\.bmp$/i)   ? &sb::Data::Image::_get_size_bmp($file)
		: ($file =~ /\.tiff?$/i) ? &sb::Data::Image::_get_size_tif($file)
		: (0,0);
	return &sb::Data::Image::_resize($w,$h,IMAGE_MAX_WIDTH,IMAGE_MAX_WIDTH);
}
# --------------------------------------
# // private functions - constants
sub _additinal_style
{
	my $self = shift;
	my $css = $self->_output(<<'__ADDITIONAL_CSS__');
	# dl#tempwiz_check dd.submit {
	# 	clear: right;
	# }
	# table#tempwiz_coltable,
	# table#tempwiz_sidetable {
	# 	width: 415px;
	# 	margin: 0px;
	# }
	# table#tempwiz_coltable td {
	# 	width: 135px;
	# }
	# table#tempwiz_coltable label img {
	# 	display: block;
	# 	margin: 0px 45px;
	# }
	# table#tempwiz_sidetable select {
	# 	width: 100px;
	# }
	# span.color_checker {
	# 	width: 1em;
	# 	height: 1em;
	# 	padding: 0px 0.3em;
	# 	border: 1px solid #000;
	# }
	# span.color_checker:hover {
	# 	cursor: pointer;
	# }
	# dl#colorform dt,
	# dl#imageform dt {
	# 	width: 160px;
	# }
	# dl#imageform dd.form_option {
	# 	margin-left: 150px;
	# }
	# dl#imageform dd.form_option label {
	# 	font-weight: normal;
	# }
	# img.selected {
	# 	background-color: blue;
	# }
	# dl#colorform dt.color {
	# 	padding: 0px;
	# }
	# dl#colorform dd.color {
	# 	padding: 0px;
	# }
	# .sample {
	# 	margin-left: 10px;
	# 	line-height: 1.4;
	# 	border: 1px dashed transparent;
	# }
	# * html .sample {
	# 	border: 0px none #000;
	# }
	# .sample:hover {
	# 	cursor: pointer;
	# 	border: 1px dashed #f00;
	# }
	# #sample_h1 {
	# 	font-weight: bold;
	# 	font-size: 120%;
	# }
	# #sample_header {
	# 	margin-bottom: 1em;
	# }
	# #sample_navigation_label {
	# 	font-weight: bold;
	# 	font-size: 90%;
	# }
	# #sample_navigation_list {
	# 	font-size: 80%;
	# }
	# #sample_entry_title_color {
	# 	font-weight: bold;
	# 	font-size: 110%;
	# }
	# #sample_entry_info {
	# 	text-align: right;
	# 	font-size: 80%;
	# }
	# #sample_body_color {
	# }
	# #sample_entry_note {
	# 	font-size: 90%;
	# }
	# #sample_link, 
	# #sample_active,
	# #sample_visited,
	# #sample_hover {
	# 	text-decoration: underline;
	# }
	# #sample_comment_title {
	# 	font-weight: bold;
	# 	font-size: 110%;
	# 	margin-top: 10px;
	# }
	# #sample_comment_label {
	# 	font-weight: bold;
	# }
	# #sample_comment_body {
	# }
	# #sample_comment_info {
	# 	text-align: right;
	# 	font-size: 80%;
	# }
	# #sample_footer {
	# 	text-align: center;
	# 	width: 450px;
	# }
	# #dhtmlgoodies_colorPicker {
	# 	position: absolute;
	# 	width: 224px;
	# 	padding-bottom: 1px;
	# 	background-color: #FFF;
	# 	border: 1px solid #317082;
	# }
	# #dhtmlgoodies_colorPicker .colorPicker_topRow {
	# 	height: 16px;
	# 	padding-bottom: 1px;
	# 	border-bottom: 3px double #317082;
	# 	background-color: #E2EBED;
	# 	padding-left: 2px;
	# 	width: 222px;
	# 	height: 16px;
	# }
	# #dhtmlgoodies_colorPicker .colorPicker_statusBar {
	# 	height: 13px;
	# 	padding-bottom: 2px;
	# 	border-top: 3px double #317082;
	# 	background-color: #E2EBED;
	# 	padding-left: 2px;
	# 	clear: both;
	# 	width: 222px;
	# 	height: 13px;
	# }
	# #dhtmlgoodies_colorPicker .colorSquare {
	# 	width: 10px;
	# 	height: 10px;
	# 	margin-left: 1px;
	# 	margin-bottom: 1px;
	# 	float: left;
	# 	border: 1px solid #000;
	# 	cursor: pointer;
	# 	width: 10px;
	# }
	# .colorPickerTab_inactive,.colorPickerTab_active {
	# 	height: 17px;
	# 	padding-left: 4px;
	# 	cursor: pointer;
	# }
	# .colorPickerTab_inactive span {
	# 	background-image: url('%image%tab_left_inactive.gif');
	# }
	# .colorPickerTab_active span {
	# 	background-image: url('%image%tab_left_active.gif');
	# }
	# .colorPickerTab_inactive span, .colorPickerTab_active span {
	# 	line-height: 16px;
	# 	font-weight: bold;
	# 	font-size: 11px;
	# 	padding-top: 1px;
	# 	vertical-align: middle;
	# 	background-position: top left;
	# 	background-repeat: no-repeat;
	# 	float: left;
	# 	padding-left: 6px;
	# }
	# .colorPickerTab_inactive img,.colorPickerTab_active img {
	# 	float: left;
	# }
	# .colorPickerCloseButton {
	# 	text-align: center;
	# 	line-height: 11px;
	# 	border: 1px solid #317082;
	# 	position: absolute;
	# 	right: 1px;
	# 	font-size: 12px;
	# 	font-weight: bold;
	# 	top: 1px;
	# 	padding: 1px;
	# 	cursor: pointer;
	# 	width: 11px;
	# 	height: 11px;
	# }
	# #colorPicker_statusBarTxt {
	# 	font-size: 11px;
	# 	vertical-align: top;
	# 	line-height: 13px;
	# }
__ADDITIONAL_CSS__
	my $path = sb::Plugin->get_resource_dir();
	$css =~ s/%image%/$path/g;
	$css;
}
sub _get_basedata_list
{
	return (
		'column',
		'calendar',
		'left',
		'center',
		'right',
		'content_width',
		'sidebar_width',
	);
}
sub _get_sidebar_list
{
	return (
		'amazon',
		'calendar',
		'selected_entry',
		'latest_entry',
		'category',
		'archives',
		'recent_comment',
		'recent_trackback',
		'profile',
		'link',
		'search',
	);
}
sub _get_color_list
{
	return (
		'body_background',
		'content_background',
		'body_color',
		'link',
		'active',
		'visited',
		'hover',
		'header',
		'h1',
		'navigation_label',
		'navigation_list',
		'footer',
		'entry_title_color',
		'entry_title_hover',
		'entry_info',
		'entry_quote',
		'entry_quote_background',
		'entry_quote_border',
		'entry_note',
		'table_color',
		'table_background',
		'td_background',
		'th_background',
		'table_border',
		'comment_title',
		'comment_label',
		'comment_body',
		'comment_info',
	);
}
sub _get_image_list
{
	return (
		'body',
		'header',
		'calendar',
		'navigation_label',
		'entry_title',
		'comment_title',
	);
}
sub _get_color_set
{
	my $self = shift;
	my $tag  = shift;
	return if (!$tag);
	my %info = (
		'body_color'             => ['556','fff',],
		'body_background'        => ['fff','000',],
		'content_background'     => ['eef','333',],
		'link'                   => ['66b','66b',],
		'active'                 => ['55c','55c',],
		'visited'                => ['96c','96c',],
		'hover'                  => ['e50','e50',],
		'header'                 => ['999','999',],
		'h1'                     => ['ddd','ddd',],
		'navigation_label'       => ['036','036',],
		'navigation_list'        => ['556','556',],
		'footer'                 => ['99c','99c',],
		'entry_title_color'      => ['036','036',],
		'entry_title_hover'      => ['369','369',],
		'entry_info'             => ['036','036',],
		'entry_quote'            => ['669','669',],
		'entry_quote_background' => ['cdd','cdd',],
		'entry_quote_border'     => ['88a','88a',],
		'entry_note'             => ['669','669',],
		'table_color'            => ['445','445',],
		'table_background'       => ['bcd','bcd',],
		'td_background'          => ['cce','cce',],
		'th_background'          => ['cdd','cdd',],
		'table_border'           => ['88a','88a',],
		'comment_title'          => ['456','456',],
		'comment_label'          => ['666','666',],
		'comment_body'           => ['557','557',],
		'comment_info'           => ['99a','99a',],
	);
	return if (!$info{$tag});
	return @{$info{$tag}};
}
sub _get_css_label
{
	my $self = shift;
	my $tag  = shift;
	return if (!$tag);
	my %info = (
		'body'                   => ['body'],
		'calendar'               => ['div.calendar'],
		'navigation_label'       => ['div.navigation dl dt'],
		'entry_title'            => ['div.entry h2.title'],
		'comment_title'          => ['div.comment h3','div.comment h4','div.trackback h3'],
		'body_color'             => ['body'],
		'body_background'        => ['body'],
		'content_background'     => ['div.wrapper'],
		'link'                   => ['a:link'],
		'active'                 => ['a:active'],
		'visited'                => ['a:visited'],
		'hover'                  => ['a:hover'],
		'header'                 => ['div.header'],
		'footer'                 => ['div.footer'],
		'h1'                     => ['h1#top','h1#top a:link','h1#top a:active','h1#top a:visited','h1#top a:hover'],
		'navigation_label'       => ['div.navigation dl dt'],
		'navigation_list'        => ['div.navigation dl dd'],
		'entry_title_color'      => ['h2.title','h2.title a:link','h2.title a:active','h2.title a:visited'],
		'entry_title_hover'      => ['h2.title a:hover'],
		'entry_info'             => ['div.entry ul.info'],
		'entry_quote'            => ['div.entry blockquote','div.entry q'],
		'entry_quote_background' => ['div.entry blockquote'],
		'entry_quote_border'     => ['div.entry blockquote'],
		'entry_note'             => ['div.entry p.note'],
		'table_color'            => ['div.entry table'],
		'table_background'       => ['div.entry table'],
		'td_background'          => ['div.entry table td'],
		'th_background'          => ['div.entry table th'],
		'table_border'           => ['div.entry table'],
		'comment_title'          => ['div.comment h3','div.comment h4','div.trackback h3'],
		'comment_label'          => ['div.comment dl dt','div.trackback dl dt'],
		'comment_body'           => ['div.comment dl.body dd','div.trackback dl.body dd'],
		'comment_info'           => ['div.comment dl dd.date','div.trackback dl dd.info'],
	);
	return if (!$info{$tag});
	return @{$info{$tag}};
}
# --------------------------------------
# // private functions - html templates
sub _template_htmlbase
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <?xml version="1.0" encoding="[site_encoding]"?>
	# <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
	# <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="[site_lang]">
	# <head>
	# <meta http-equiv="Content-Type" content="text/html; charset=[site_encoding]" />
	# <meta http-equiv="Content-Script-Type" content="text/javascript" />
	# <meta http-equiv="Content-Style-Type" content="text/css" />
	# <link rel="stylesheet" href="[site_css]" type="text/css" />
	# <link rel="alternate" type="application/rss+xml" title="RSS" href="[site_rss]" />
	# <link rel="alternate" type="application/atom+xml" href="[site_atom]" />
	# <link rel="EditURI" type="application/rsd+xml" title="RSD" href="[site_rsd]" />
	# <title>[site_title]</title>
	# </head>
	# <body>
	# <div class="wrapper">
	# <div class="header">
	# <!--%BEGIN title%-->
	# <h1 id="top">[blog_name]</h1>
	# <p>[blog_description]</p>
	# <!--%END title%-->
	# <!--_end_header_--></div>
	# <div class="body">
	# {tempwiz_horizontal_calendar}{tempwiz_subnavi}<div class="contents">
	# <!--%BEGIN entry%-->
	# <div class="entry">
	# <h2 class="title"><a href="[entry_permalink]">[entry_title]</a></h2>
	# <ul class="info">
	# <li class="cat">[category_name]</li>
	# <li class="auth">by [user_name]</li>
	# <li class="date">[entry_date] [entry_disp_time]</li>
	# </ul>
	# <div class="entry_body">[entry_description]</div>
	# <div class="entry_more">[entry_sequel]</div>
	# <ul class="status">
	# <li class="com">[comment_num]</li>
	# <li class="tb">[trackback_num]</li>
	# </ul>
	# <!--_end_entry_--></div>
	# [trackback_auto_discovery]
	# <!--%END entry%-->
	# <!--%BEGIN comment_area%-->
	# <div class="comment">
	# <h3 id="comments">Comments</h3>
	# <!--%BEGIN comment%-->
	# <dl class="body">
	# <dt>[comment_name] [comment_icon]</dt>
	# <dd>[comment_description]</dd>
	# <dd class="date">[comment_time]</dd>
	# </dl>
	# <!--%END comment%-->
	# <form action="[site_cgi]" method="post">
	# <h4>Comment Form</h4>
	# <dl>
	# <dt><label for="name" id="labelname">name:</label></dt>
	# <dd>
	# <input type="text" tabindex="1" name="name" id="name" value="[cookie_name]" size="20" />
	# <label for="set_cookie">Cookie</label><input type="checkbox" name="set_cookie" value="1" id="set_cookie">
	# </dd>
	# <dt><label for="email" id="labelemail">email:</label></dt>
	# <dd><input type="text" tabindex="2" name="email" id="email" value="[cookie_email]" size="30" /></dd>
	# <dt><label for="url" id="labelurl">url:</label></dt>
	# <dd><input type="text" tabindex="3" name="url" id="url" value="[cookie_url]" size="30" /></dd>
	# <dt><label for="description" id="labeldescription">comments:</label></dt>
	# <dd><textarea tabindex="5" id="description" name="description" rows="8" cols="45"></textarea></dd>
	# <dt><label for="description" id="labelsubmit">submit</label></dt>
	# <dd>
	# <input type="submit" tabindex="6" id="submit" value="submit" />
	# <input type="reset" value="reset" />
	# <input type="hidden" name="entry_id" value="[entry_id]" />
	# <input type="hidden" name="charset" value="[site_encoding]" />
	# </dd>
	# </dl>
	# </form>
	# <!--_end_comment_--></div>
	# <!--%END comment_area%-->
	# <!--%BEGIN trackback_area%-->
	# <div class="trackback">
	# <h3 id="trackback">Trackbacks</h3>
	# <p class="link">Trackback URL : <strong>[trackback_url]</strong></p>
	# <!--%BEGIN trackback%-->
	# <dl class="body">
	# <dt>[trackback_title]</dt>
	# <dd>[trackback_excerpt]</dd>
	# <dd class="info">[trackback_blog_name] | [trackback_time]</dd>
	# </dl>
	# <!--%END trackback%-->
	# <!--_end_trackback_--></div>
	# <!--%END trackback_area%-->
	# <!--%BEGIN profile_area%-->
	# <div class="entry">
	# <h2 class="title">[profile_name]</h2>
	# <ul class="info">
	# <li class="cat">user profile</li>
	# </ul>
	# <div class="entry_body">[profile_description]</div>
	# <!--_end_entry_--></div>
	# <!--%END profile_area%-->
	# <!--%BEGIN page%-->
	# <div class="page">
	# [prev_page_link] [page_now]/[page_num] [next_page_link]
	# </div>
	# <!--%END page%-->
	# <!--%BEGIN sequel%-->
	# <div class="page">
	# [prev_entry] | [next_entry]
	# </div>
	# <!--%END sequel%-->
	# <!--_end_contents_--></div>
	# {tempwiz_mainnavi}<hr id="body_separator" />
	# <!--_end_body_--></div>
	# <div class="footer">
	# <address>Copyright &copy; 2006 <a href="[script_webpage]">SimpleBoxes</a>. All Rights Reserved.</address>
	# <p id="link2top"><a href="#top">page top</a></p>
	# <!--%BEGIN counter%-->
	# [collect_log]
	# <!--%END counter%-->
	# <!--_end_footer_--></div>
	# </div>
	# </body>
	# </html>
__END_PARTS__
}
sub _template_amazon
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN amazon%-->
	# <dl class="recommend">
	# <dt class="label">Recommendation</dt>
	# <dd class="imag"><a href="[amazon_url]"><img src="[amazon_ImageUrlSmall]" alt="[amazon_ProductName]" class="amazon_pict" /></a></dd>
	# <dd class="name"><a href="[amazon_url]">[amazon_ProductName]</a></dd>
	# </dl>
	# <!--%END amazon%-->
__END_PARTS__
}
sub _template_calendar
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN calendar%-->
	# <dl class="calendar">
	# <dt id="calendarname">Calendar</dt>
	# <dd id="calendarlist">[calendar2]</dd>
	# </dl>
	# <!--%END calendar%-->
__END_PARTS__
}
sub _template_selected_entry
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN selected_entry%-->
	# <dl class="selectedentry">
	# <dt id="selectedentryname">Selected articles</dt>
	# <dd id="selectedentrylist">[selected_entry_list]</dd>
	# </dl>
	# <!--%END latest_entry%-->
__END_PARTS__
}
sub _template_latest_entry
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN latest_entry%-->
	# <dl class="recententry">
	# <dt id="newentryname">Latest articles</dt>
	# <dd id="newentrylist">[latest_entry_list]</dd>
	# </dl>
	# <!--%END latest_entry%-->
__END_PARTS__
}
sub _template_category
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN category%-->
	# <dl class="category">
	# <dt id="categoryname">Categories</dt>
	# <dd id="categorylist">[category_list]</dd>
	# </dl>
	# <!--%END category%-->
__END_PARTS__
}
sub _template_archives
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN archives%-->
	# <dl class="archives">
	# <dt id="archivesname">Archives</dt>
	# <dd id="archiveslist">[archives_list]</dd>
	# </dl>
	# <!--%END archives%-->
__END_PARTS__
}
sub _template_recent_comment
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN recent_comment%-->
	# <dl class="recentcomment">
	# <dt id="commentname">Comments</dt>
	# <dd id="commentlist">[recent_comment_list]</dd>
	# </dl>
	# <!--%END recent_comment%-->
__END_PARTS__
}
sub _template_recent_trackback
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN recent_trackback%-->
	# <dl class="recenttrackback">
	# <dt id="tbname">Trackbacks</dt>
	# <dd id="tblist">[recent_trackback_list]</dd>
	# </dl>
	# <!--%END recent_trackback%-->
__END_PARTS__
}
sub _template_profile
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN profile%-->
	# <dl class="profile">
	# <dt id="profilename">Users</dt>
	# <dd id="profilelist">[user_list]</dd>
	# </dl>
	# <!--%END profile%-->
__END_PARTS__
}
sub _template_link
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN link%-->
	# <dl class="links">
	# <dt id="linkname">Links</dt>
	# <dd id="linklist">[link_list]</dd>
	# </dl>
	# <!--%END link%-->
__END_PARTS__
}
sub _template_search
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <form method="get" action="[site_cgi]">
	# <dl class="search">
	# <dt>Search</dt>
	# <dd><input type="text" id="search" name="search" size="16" class="form" /><input type="submit" value="Search" class="button" /></dd>
	# </dl>
	# </form>
__END_PARTS__
}
sub _template_other
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <dl class="extra">
	# <dt id="extraname">Others</dt>
	# <dd id="extralist">
	# <ul>
	# <li><a href="[site_rss]">RSS1.0</a></li>
	# <li><a href="[site_atom]">Atom0.3</a></li>
	# <li><a href="[script_webpage]">[script_name] [script_version]</a></li>
	# </ul>
	# </dd>
	# </dl>
__END_PARTS__
}
sub _template_horizontal_calendar
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# <!--%BEGIN calendar%-->
	# <div class="calendar">[calendar_horizontal]</div>
	# <!--%END calendar%-->
__END_PARTS__
}
sub _template_navi
{
	my $self = shift;
	my $navi_name = shift;
	my $navi_content = shift;
	return $self->_output(<<"__END_PARTS__");
	# <div class="navigation" id="$navi_name">
	# $navi_content<!--_end_navigation_--></div>
__END_PARTS__
}
# --------------------------------------
# // private functions - css templates
sub _template_cssbase
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# /* style.css */
	# /* == written by Takuya Otani <takuya.otani@gmail.com> === */
	# /* == Copyright (C) 2004 SimpleBoxes/SerendipityNZ Ltd. == */
	# 
	# /* [defaults] */
	# * {
	# 	margin: 0px;
	# 	padding: 0px;
	# }
	# h1, h2, h3, h4, h5, h6 {
	# 	margin: 1em 0px;
	# 	font-weight: bold;
	# 	font-style: normal;
	# 	font-family: sans-serif;
	# }
	# p, dl, ul, ol, blockquote, form, pre {
	# 	font-weight: normal;
	# 	font-style: normal;
	# }
	# th, dt {
	# 	font-weight: bold;
	# 	font-style: normal;
	# }
	# strong {
	# 	font-weight: bold;
	# 	font-style: normal;
	# }
	# em {
	# 	font-weight: normal;
	# 	font-style: normal;
	# }
	# img {
	# 	border: none;
	# }
	# q {
	# 	font-style: normal;
	# 	quotes: none;
	# }
	# a {
	# 	text-decoration: underline;
	# }
	# /* [blocks] */
	# body {
	# 	margin: 0px;
	# 	padding: 0px;
	# 	text-align: center;
	# }
	# div.wrapper {
	# 	width: {body_width}px;
	# 	margin: 0px;
	# 	margin-left: auto;
	# 	margin-right: auto;
	# 	text-align: left;
	# }
	# div.body, 
	# div.header, 
	# div.footer {
	# 	clear: both;
	# 	margin: 0px;
	# 	padding: 0px;
	# }
	# hr#body_separator {
	# 	clear: both;
	# 	height: 20px;
	# 	visibility: hidden;
	# }
	# /* [header] */
	# div.header {
	# 	margin-top: 10px;
	# 	height: 100px;
	# 	text-align: right;
	# }
	# div.header h1#top {
	# 	margin: 0px;
	# 	padding: 10px 20px;
	# 	font-size: 20px;
	# 	text-align: left;
	# }
	# div.header h1#top a {
	# 	text-decoration: none;
	# }
	# div.header h1#top a:hover {
	# 	text-decoration: underline;
	# }
	# div.header p {
	# 	margin: 0px;
	# 	padding: 0px 20px;
	# 	font-size: 10px;
	# 	text-align: left;
	# }
	# /* [articles] */
	# div.entry {
	# 	margin-left: 10px;
	# 	padding-right: 10px;
	# }
	# div.entry div.entry_body,
	# div.entry div.entry_more {
	# 	margin: 0px 10px;
	# 	line-height: 1.4;
	# }
	# /* [elements in article] */
	# div.entry p,
	# div.entry ul,
	# div.entry ol,
	# div.entry dl,
	# div.entry table,
	# div.entry blockquote,
	# div.entry h1,
	# div.entry h2,
	# div.entry h3,
	# div.entry h4,
	# div.entry h5,
	# div.entry h6,
	# div.entry pre {
	# 	margin: 1em 0px;
	# 	padding: 0px;
	# 	line-height: 1.4;
	# }
	# div.entry ul,
	# div.entry ol,
	# div.entry dl,
	# div.entry table,
	# div.entry blockquote,
	# div.entry h1,
	# div.entry h2,
	# div.entry h3,
	# div.entry h4,
	# div.entry h5,
	# div.entry h6,
	# div.entry pre {
	# 	clear: left;
	# }
	# div.entry li,
	# div.entry dd {
	# 	margin-left: 2em;
	# }
	# div.entry blockquote {
	# 	padding: 2px 10px;
	# 	font-style: italic;
	# }
	# div.entry blockquote p {
	# 	padding: 0px;
	# 	margin: 0px;
	# }
	# div.entry table th {
	# 	margin: 0px;
	# 	padding: 5px;
	# 	text-align: center;
	# }
	# div.entry table td {
	# 	margin: 0px;
	# 	padding: 2px;
	# }
	# div.entry p.note {
	# 	font-size: 90%;
	# }
	# div.entry q {
	# 	quotes: "";
	# 	font-style: italic;
	# }
	# div.entry cite {
	# 	font-style: normal;
	# }
	# div.entry ins {
	# 	text-decoration: none;
	# }
	# div.entry del {
	# 	text-decoration: line-through;
	# }
	# div.entry img.pict {
	# 	display: block;
	# 	text-align: center;
	# 	margin: auto;
	# }
	# div.entry div.amazon {
	# 	text-align: center;
	# }
	# div.entry div.amazon div.amazon_text {
	# 	text-align: left;
	# }
	# /* [special elements in article] */
	# div.entry h2.title {
	# 	margin: 20px -10px 0px 0px;
	# 	padding: 6px 0px 30px 40px;
	# 	line-height: 1.0;
	# 	font-size: 14px;
	# }
	# div.entry h2.title a {
	# 	text-decoration: none;
	# }
	# div.entry ul.info,
	# div.entry ul.status {
	# 	margin: 0px;
	# 	text-align: right;
	# 	font-size: 11px;
	# 	list-style: none;
	# }
	# div.entry ul.info {
	# 	position: relative;
	# 	top: -20px;
	# }
	# div.entry ul.status {
	# 	margin: 0px;
	# 	padding: 0px 10px;
	# }
	# div.entry ul.info li,
	# div.entry ul.status li {
	# 	margin: 0px;
	# 	display: inline;
	# }
	# div.entry ul.info li {
	# 	line-height: 1.0;
	# 	padding-left: 0.5em;
	# }
	# div.entry ul.status li.com {
	# 	padding-right: 5px;
	# 	margin-right: 5px;
	# }
	# div.entry ul.info a,
	# div.entry ul.status a {
	# 	text-decoration: none;
	# }
	# div.entry ul.info a:hover,
	# div.entry ul.status a:hover {
	# 	text-decoration: underline;
	# }
	# /* [page navi] */
	# div.page {
	# 	padding: 10px 0px;
	# 	text-align: center;
	# 	font-size: 11px;
	# }
	# div.page a {
	# 	text-decoration: none;
	# }
	# /* [comments/trackbacks] */
	# div.trackback {
	# 	margin-top: 20px;
	# }
	# div.comment h3,
	# div.comment h4,
	# div.trackback h3 {
	# 	margin: 0px 10px 5px 10px;
	# 	padding: 15px 0px 0px 20px;
	# 	font-size: 14px;
	# }
	# div.comment dl,
	# div.trackback dl {
	# 	margin-left: 20px;
	# 	padding-right: 20px;
	# 	line-height: 1.4;
	# 	font-size: 90%;
	# }
	# div.comment dl dd,
	# div.trackback dl dd {
	# 	margin-left: 15px;
	# 	padding: 0px;
	# }
	# div.comment dl dd.date,
	# div.trackback dl dd.info
	# {
	# 	text-align: right;
	# 	font-size: 90%;
	# }
	# div.comment dl.body dt,
	# div.trackback dl.body dt {
	# }
	# div.trackback p.link {
	# 	margin: 0px 0px 0px 10px;
	# 	padding-right: 10px;
	# 	text-align: right;
	# 	font-size: 90%;
	# }
	# div.comment form dl dt {
	# 	display: block;
	# 	clear: left;
	# 	float: left;
	# 	width: 8em;
	# 	margin: 0px;
	# 	padding: 0px;
	# 	text-align: right;
	# }
	# div.comment form dl dt label#labelsubmit {
	# 	display: none;
	# }
	# div.comment form dl dd {
	# 	margin: 0px 0px 0px 8.5em;
	# 	padding: 0px;
	# }
	# /* [calendar] */
	# div.calendar {
	# 	clear: both;
	# 	padding: 10px 0px;
	# 	font-size: 11px;
	# 	text-align: center;
	# }
	# div.calendar a {
	# 	font-weight: bold;
	# 	text-decoration: none;
	# }
	# /* [footer] */
	# div.footer {
	# 	margin-bottom: 0px;
	# }
	# div.footer address {
	# 	font-weight: bold;
	# 	font-style: italic;
	# }
	# div.footer address,
	# div.footer p#link2top {
	# 	margin: 0px;
	# 	padding-bottom: 10px;
	# 	text-align: center;
	# 	line-height: 1.0;
	# 	font-size: 11px;
	# }
	# div.footer address a {
	# 	text-decoration: none;
	# }
	# /* [navigation] */
	# div.navigation dl {
	# 	width: {navi_width}px;
	# 	padding: 0px;
	# }
	# div.navigation dl {
	# 	padding-top: 10px;
	# }
	# div.navigation ul {
	# 	list-style: none;
	# 	margin: 0px;
	# 	padding: 0px;
	# }
	# div.navigation ul a {
	# 	text-decoration: none;
	# }
	# div.navigation dl dt {
	# 	margin: 0px;
	# 	padding: 9px 5px 9px 32px;
	# 	line-height: 1.0;
	# 	font-size: 12px;
	# 	font-weight: bold;
	# }
	# div.navigation dl dd {
	# 	margin: 0px;
	# 	padding: 0px 0px 5px 5px;
	# }
	# div.navigation dl.recommend dd.imag {
	# 	text-align: center;
	# }
	# div.navigation dl.recommend dd a {
	# 	text-decoration: none;
	# }
	# div.navigation dl dd ul li {
	# 	padding: 2px 0px;
	# 	font-size: 11px;
	# }
	# div.navigation dl dd ul li ul li {
	# 	padding-left: 1em;
	# }
	# div.navigation dl.extra dt {
	# 	display: none;
	# }
	# div.navigation dl.extra dd {
	# 	margin-top: 1em;
	# }
	# div.navigation dl.extra ul li,
	# div.navigation dl.search dd {
	# 	padding: 3px 0px;
	# 	text-align: center;
	# 	line-height: 1.3;
	# }
	# /* [calendar] */
	# div.navigation table.calendar {
	# 	width: {navi_width}px;
	# }
	# div.navigation table.calendar td {
	# 	padding: 2px 0px;
	# 	margin: 0px;
	# 	text-align: center;
	# 	font-size: 10px;
	# 	line-height: 1.0;
	# }
	# div.navigation table.calendar td.weekday {
	# 	width: {cal_width}px;
	# }
	# div.navigation table.calendar td.calendar_month {
	# 	padding-bottom: 0px 0px 4px 0px;
	# 	font-weight: bold;
	# }
	# div.navigation table.calendar td.calendar_month a {
	# 	text-decoration: none;
	# }
	# div.navigation table.calendar td.cell {
	# 	padding: 1px 0px;
	# }
	# div.navigation table.calendar td.cell a {
	# 	font-weight: bold;
	# 	text-decoration: none;
	# }
	# div.navigation table.calendar td.cell span.today {
	# 	font-weight: bold;
	# }
	# /* [fonts] */
	# h1, h2, h3, h4, h5, h6, p, ul, ol, dl, li, dd, dt, td, th, address, blockquote {
	# 	/* except MacIE \*/
	# 	font-family: "Lucida Grande", sans-serif!important;
	# 	/* end for MacIE */
	# }
	# h1, h2, h3, h4, h5, h6, th, dt, strong {
	# 	font-family: "Hiragino Kaku Gothic Std",sans-serif;
	# }
	# pre, code, kbd {
	# 	font-family: monospace;
	# }
	# div.navigation table.calendar td.weekday {
	# 	font-family: monospace!important;
	# }
	# address {
	# 	font-family: "Times","Times New Roman",serif!important;
	# }
	# div.header h1,
	# div.calendar {
	# 	font-family: "Times","Times New Roman","Hiragino Mincho Std","MS PMincho",serif!important;
	# }
	# div.calendar a,
	# div.calendar span.calendar_month {
	# 	font-family: "Verdana",sans-serif!important;
	# }
	# div.navigation dl dt {
	# 	font-family: "Arial","Trebuchet MS","Trebuchet",sans-serif!important;
	# }
	# div.comment h3,
	# div.comment h4,
	# div.trackback h3,
	# div.trackback p.link {
	# 	font-family: "Trebuchet MS","Trebuchet",sans-serif!important;
	# }
	# div.comment dl.form dt {
	# 	font-family: "Verdana",sans-serif!important;
	# }
__END_PARTS__
}
sub _template_css_2columns
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# /* [layout] */
	# div.contents {
	# 	float: {content_float};
	# 	width: {content_width}px;
	# 	margin: 0px;
	# }
	# div.navigation {
	# 	margin-{navi_float}: {navi_margin}px;
	# }
	# * html div.navigation {
	# 	margin-{navi_float}: {navi_margin_ie}px;
	# }
__END_PARTS__
}
sub _template_css_3columns
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# /* [layout] */
	# div.contents {
	# 	float: left;
	# 	width: {content_width}px;
	# }
	# div#navi1st {
	# 	float: left;
	# 	width: {navi_width}px;
	# }
	# div#navi2nd {
	# 	width: {navi_width}px;
	# 	margin-left: {navi_margin}px;
	# }
	# * html div#navi2nd {
	# 	float: left;
	# 	margin-left: 0px;
	# }
__END_PARTS__
}
sub _template_css_color
{
	my $self = shift;
	my $tag  = shift;
	return if (!$tag);
	my $data = '';
	if ($tag =~ /border/ and $tag eq 'table_border')
	{
		$data = $self->_output(<<'__END_PARTS__');
		# %css_tag% {
		# 	border-right: 1px solid #%color_code%;
		# 	border-bottom: 1px solid #%color_code%;
		# }
		# div.entry table td,
		# div.entry table th {
		# 	border-top: 1px solid #%color_code%;
		# 	border-left: 1px solid #%color_code%;
		# }
__END_PARTS__
	}
	elsif ($tag =~ /border/)
	{
		$data = $self->_output(<<'__END_PARTS__');
		# %css_tag% {
		# 	border: 1px solid #%color_code%;
		# }
__END_PARTS__
	}
	elsif ($tag =~ /background/ and $tag eq 'content_background')
	{
		$data = $self->_output(<<'__END_PARTS__');
		# %css_tag% {
		# 	background-color: #%color_code%;
		# 	border: 1px solid #%color_code%;
		# }
__END_PARTS__
	}
	elsif ($tag =~ /background/)
	{
		$data = $self->_output(<<'__END_PARTS__');
		# %css_tag% {
		# 	background-color: #%color_code%;
		# }
__END_PARTS__
	}
	else
	{
		$data = $self->_output(<<'__END_PARTS__');
		# %css_tag% {
		# 	color: #%color_code%;
		# }
__END_PARTS__
	}
	return $data;
}
sub _template_css_image
{
	my $self = shift;
	my $tag  = shift;
	return if (!$tag);
	return $self->_output(<<'__END_PARTS__');
	# %css_tag% {
	# 	background-position: %img_x% %img_y%;
	# 	background-repeat: %img_repeat%;
	# 	background-image: url({site_parts}%img_name%);
	# }
__END_PARTS__
}
sub _template_info
{
	my $self = shift;
	return $self->_output(<<'__END_PARTS__');
	# This template is created by "Template Wizard".
	# 
	# Template Wizard (C) 2007 SimpleBoxes/SerendipityNZ
__END_PARTS__
}
1;
__END__
